# 
###############################################################################
#Programmer: Orcan Ogetbil    orcan@nbcs.rutgers.edu                          #
#Date: 07/02/2010                                                             #
#Filename: genrepo.py                                                         #
#                                                                             #
#                                                                             #
#       Copyright 2010 Orcan Ogetbil                                          #
#                                                                             #
#    This program is free software; you can redistribute it and/or modify     #
#    it under the terms of the GNU General Public License as published by     #
#    the Free Software Foundation; either version 2 of the License, or        #
#    (at your option) any later version.                                      #
#                                                                             #
#    This program is distributed in the hope that it will be useful,          #
#    but WITHOUT ANY WARRANTY; without even the implied warranty of           #
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            #
#    GNU General Public License for more details.                             #
#                                                                             #
###############################################################################
#
# Library to generate rutgers rpm repositories

import koji
import os
import errno
import shutil
import sys
import string
import tempfile
import logging
import getpass
import imp
sys.path.append('/usr/share/createrepo/')
sys.modules['createrepo'] = imp.load_source("createrepo","/usr/share/createrepo/genpkgmetadata.py")
import createrepo

import rcommon

config = rcommon.loadConfig()
hub = config.get('koji','hub')

koji_pkg_dir = config.get('koji','pkgdir')
repos_dir = config.get('repositories', 'repodir_private')
debugrepo = config.get('repositories', 'debugrepo')

relver = config.get('repositories', 'distver')
relname = config.get('repositories', 'distname')
archs = config.get('repositories', 'archs').split()
repo_prefix = relname + relver + "-"


def mkdir_p(path):
	os.umask(002)
	try:
		os.makedirs(path)
	except OSError, exc:
		if exc.errno == errno.EEXIST:
			pass
		else: raise

def genRepos(repos, builddebug=False):
	global my_logger
	
	theSuspect = config.get("report","user")+"-"+getpass.getuser()
	my_logger = logging.getLogger(theSuspect)
	
	os.umask(002)
	tempprefix = 'tmprepodir-'
	repos_tmpdir = tempfile.mkdtemp(prefix=tempprefix)
	kojisession = koji.ClientSession(hub)
	if builddebug:
		createDebugRepo(kojisession, repos_tmpdir)
		my_logger.info("All done creating the debug repo. Now replace old repo with the new one.")
		try:
			shutil.rmtree(repos_dir + "/" + debugrepo)
		except OSError, exc:
			if exc.errno == errno.ENOENT:
				pass
			else: raise
		shutil.move(repos_tmpdir + "/debug", repos_dir + "/" + debugrepo)
		
	if repos:
		for repo in repos:
			createRepo(kojisession, repo, repos_tmpdir)
		my_logger.info("All done creating repos. Now replace old repos with the new ones.")
		for repo in repos:
			try:
				shutil.rmtree(repos_dir + "/" + repo)
			except OSError, exc:
				if exc.errno == errno.ENOENT:
					pass
				else: raise
			shutil.move(repos_tmpdir + "/" + repo, repos_dir + "/" + repo)
	
	my_logger.info("Cleaning up.")
	shutil.rmtree(repos_tmpdir, ignore_errors = True)


def createDebugRepo(kojisession, repos_tmpdir):
	repo_tmpdir = repos_tmpdir + "/debug"
	my_logger.info("Creating temporary debug repo at " + repo_tmpdir)
	fresh = False
	for arch in archs:
		mkdir_p(repo_tmpdir + "/" + arch)
		try:
			shutil.copytree(repos_dir + "/" + debugrepo + "/" + arch + "/repodata", repo_tmpdir + "/" + arch + "/repodata")
		except OSError:
			fresh = True
	if fresh:
		my_logger.warning("No previous record of the debug repo. Creating from scratch.")
	else:
		my_logger.info("We will use the old debug repo metadata to speed things up.")

	repostodebug = config.get("repositories", "repostodebug").split()
	latestRPMS = []
	for repo in repostodebug:
		my_logger.info("Retrieving the list of latest packages for repo: " + repo)
		latestRPMS += kojisession.getLatestRPMS(repo_prefix+repo)[0]

	my_logger.info("Populating debug repo with debuginfo RPMs from: " + string.join(repostodebug, " "))
	for rpm in latestRPMS:
		if string.find(rpm['name'], "-debuginfo") == -1:
			continue
		
		build = kojisession.getBuild(rpm['build_id'])
		nvr_dir = koji_pkg_dir + build['name'] + "/" + build['version'] + "/" + build['release']
		filename = rpm['name'] + "-" + rpm['version'] + "-" + rpm['release'] + "." + rpm['arch'] + ".rpm"
		filepath = nvr_dir + "/" + rpm['arch'] + "/" + filename

		try:
			os.symlink(filepath, repo_tmpdir + "/" + rpm['arch'] + "/" + filename)
		except OSError, exc:
			# Same RPM might exist in multiple repos (stable, testing)
			if exc.errno == errno.EEXIST:
				pass
			else: raise
				
	for archdir in archs:
		if fresh:
			createrepo.main([repo_tmpdir + "/" + archdir])
		else:
			createrepo.main(["--update", repo_tmpdir + "/" + archdir])



def createRepo(kojisession, repo, repos_tmpdir):
	repo_dir = repos_dir + "/" + repo + "/" + relver
	repo_tmpdir = repos_tmpdir + "/" + repo + "/" + relver
	my_logger.info("Creating temporary repo at " + repo_tmpdir)
	fresh = False
	for arch in archs + ["SRPMS"]:
		mkdir_p(repo_tmpdir + "/" + arch)
		try:
			shutil.copytree(repo_dir + "/" + arch + "/repodata", repo_tmpdir + "/" + arch + "/repodata")
		except OSError:
			fresh = True
	if fresh:
		my_logger.warning("No previous record of this repo. Creating from scratch.")
	else:
		my_logger.info("We will use the old repo metadata to speed things up.")

	my_logger.info("Retrieving the list of latest packages for repo: " + repo)
	latestRPMS = kojisession.getLatestRPMS(repo_prefix+repo)[0]

	my_logger.info("Populating repo: " + repo)
	for rpm in latestRPMS:
		if string.find(rpm['name'], "-debuginfo") > 0:
			# These are handled elsewhere
			continue
		taginfo = kojisession.listTags(rpm['build_id'])
		tags = []
		for info in taginfo:
			tags.append(info['name'])
		if not repo_prefix+repo in tags:
			continue

		build = kojisession.getBuild(rpm['build_id'])
		nvr_dir = koji_pkg_dir + build['name'] + "/" + build['version'] + "/" + build['release']
		filename = rpm['name'] + "-" + rpm['version'] + "-" + rpm['release'] + "." + rpm['arch'] + ".rpm"
		filepath = nvr_dir + "/" + rpm['arch'] + "/" + filename

		if rpm['arch'] == 'noarch':
			for arch in archs:
				os.symlink(filepath, repo_tmpdir + "/" + arch + "/" + filename)
		elif rpm['arch'] == 'src':
			os.symlink(filepath, repo_tmpdir + "/SRPMS/" + filename)
		else:
			os.symlink(filepath, repo_tmpdir + "/" + rpm['arch'] + "/" + filename)

	for archdir in archs + ["SRPMS"]:
		if fresh:
			createrepo.main([repo_tmpdir + "/" + archdir])
		else:
			createrepo.main(["--update", repo_tmpdir + "/" + archdir])


